@@ -5,17 +5,17 @@
  *
  * Author: Neels Hofmeyr
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: AGPL-3.0+
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  */
 
 #include <osmocom/core/fsm.h>
@@ -380,7 +380,7 @@ static void msc_ho_send_handover_request(struct msc_a *msc_a)
 	struct vlr_subscr *vsub = msc_a_vsub(msc_a);
 	struct gsm_network *net = msc_a_net(msc_a);
 	struct gsm0808_channel_type channel_type;
-	struct gsm0808_speech_codec_list scl;
+	struct gsm0808_speech_codec_list scl = {};
 	struct gsm_trans *cc_trans = msc_a->cc.active_trans;
 	struct ran_msg ran_enc_msg = {
 		.msg_type = RAN_MSG_HANDOVER_REQUEST,
@@ -422,6 +422,8 @@ static void msc_ho_send_handover_request(struct msc_a *msc_a)
 				return;
 			}
 			break;
+		case GSM48_BCAP_ITCAP_3k1_AUDIO:
+		case GSM48_BCAP_ITCAP_FAX_G3:
 		case GSM48_BCAP_ITCAP_UNR_DIG_INF:
 			if (csd_bs_list_to_gsm0808_channel_type(&channel_type, &cc_trans->cc.local.bearer_services)) {
 				msc_ho_failed(msc_a, GSM0808_CAUSE_EQUIPMENT_FAILURE,
@@ -440,7 +442,13 @@ static void msc_ho_send_handover_request(struct msc_a *msc_a)
 		ran_enc_msg.handover_request.call_id_present = true;
 		ran_enc_msg.handover_request.call_id = cc_trans->call_id;
 
-		sdp_audio_codecs_to_speech_codec_list(&scl, &cc_trans->cc.local.audio_codecs);
+		/* Call assignment is now capable of re-assigning to overcome a codec mismatch with the remote call leg.
+		 * But for inter-MSC handover, that is not supported yet. So keep here the old limitation of only
+		 * offering the assigned codec. */
+		if (sdp_audio_codec_is_set(&cc_trans->cc.codecs.assignment))
+			sdp_audio_codec_to_speech_codec_list(&scl, &cc_trans->cc.codecs.assignment);
+		else
+			sdp_audio_codecs_to_speech_codec_list(&scl, &cc_trans->cc.local.audio_codecs);
 		if (!scl.len) {
 			msc_ho_failed(msc_a, GSM0808_CAUSE_EQUIPMENT_FAILURE, "Failed to compose"
 				      " Codec List (MSC Preferred) for Handover Request message\n");
